@@ -14,6 +14,8 @@ import yaml
 import json
 import os
 import re
+import asyncio
+from typing import Dict, Any, List, Optional
 
 from .utils.config import INTERFACES, INTERFACES_WITH_INDEX, BROWSER_OPTIONS, DEFAULT_TIMEOUT, KEYBOARD
 from .utils.functions import required_fields, is_valid_url, action_name
@@ -116,9 +118,10 @@ class PyWebWizardBase:
     and defines basic actions like clicking or filling fields.
     """
 
-    def __init__(self, spell_file: str, headers: dict = None, destruct_on_finish: bool = True,
-                 external_functions_mapping: dict = None, keepass_file: str = None, keepass_password: str = None,
-                 undetectable: bool = False):
+    def __init__(self, spell_file: str = None, spell_book: list = None, headers: dict = None,
+                 destruct_on_finish: bool = True, external_functions_mapping: dict = None,
+                 keepass_file: str = None, keepass_password: str = None,
+                 undetectable: bool = False, openai_api_key: str = None, ai_model: str = "gpt-4"):
         """
         Initializes the PyWebWizardBase instance.
 
@@ -132,10 +135,11 @@ class PyWebWizardBase:
         self.env_var_separator_end = "}}"
         self.actions_map = {}
         self.headers = headers
-        self.actions, self.config, self.environment = self.load_spell(spell_file)
-        if not self.config:
-            logging.error("[CONFIG] Config not found")
-            raise ConfigFileError("Config not found")
+        self.actions = []
+        self.config = {}
+        self.environment = {}
+        if spell_file:
+            self.actions, self.config, self.environment = self.load_spell(spell_file)
         self.destruct_on_finish = self.config.get('destroy', destruct_on_finish) is True
         self.external_functions_mapping = external_functions_mapping
         self.prepare_screenshots_dir()
@@ -145,7 +149,17 @@ class PyWebWizardBase:
         self.keepass_file = keepass_file
         self.keepass_password = keepass_password
         self.keepass_db = None
-        self.undetectable = self.config.get('undetectable')  # Añadido para controlar el modo
+        self.undetectable = self.config.get('undetectable')
+        self.ai_enabled = bool(openai_api_key)
+        if self.ai_enabled:
+            try:
+                from .utils.ai_controller import AIController
+                self.ai = AIController(openai_api_key, ai_model)
+                logging.info("AI enabled successfully")
+            except ImportError as e:
+                self.ai_enabled = False
+                logging.warning(f"Could not load AI module: {str(e)}")
+                logging.warning("AI functionality will be disabled")
         if keepass_file and keepass_password:
             if os.path.isfile(keepass_file):
                 try:
@@ -166,6 +180,37 @@ class PyWebWizardBase:
                 action_name = attr._action_name
                 self.actions_map[action_name] = attr
 
+    async def invoke_ai(self, task_description: str) -> bool:
+        """
+        Executes a complex task using AI to determine the necessary actions.
+
+        :param task_description: Description of the task to be performed
+        :return: True if the task was completed successfully, False otherwise
+        """
+        if not self.ai_enabled:
+            logging.error("AI functionality is not enabled. Please provide an OpenAI API key.")
+            return False
+
+        try:
+            current_page_info = self._html({})
+            logging.info(f"Analyzing task: {task_description}")
+            actions = await self.ai.analyze_task(task_description, current_page_info)
+
+            if not actions:
+                logging.warning("No actions were generated for the task")
+                return False
+
+            logging.info(f"Executing {len(actions)} AI-generated actions")
+            for i, action in enumerate(actions, 1):
+                logging.info(f"Executing action {i}/{len(actions)}: {action.get('action', 'unknown')}")
+                self._execute_action(action)
+
+            return True
+
+        except Exception as e:
+            logging.error(f"Error executing AI task: {str(e)}")
+            return False
+
     def invoke(self) -> list:
         """
         Starts the automated browser actions based on the loaded configuration.
@@ -326,37 +371,53 @@ class PyWebWizardBase:
         """
         Sets up the browser driver based on the configuration settings.
         """
-        if self.config.get('browser') not in BROWSER_OPTIONS:
+        if self.config.get('browser', 'chrome') not in BROWSER_OPTIONS:
+            logging.error(f"[DRIVER] Browser not supported: {self.config.get('browser', 'chrome')}")
+            logging.error(f"[DRIVER] Available browsers: {BROWSER_OPTIONS.keys()}")
+            logging.error(f"[DRIVER] Config: {self.config}")
             raise BrowserError("Browser not supported")
 
-        _browser_options = BROWSER_OPTIONS.get(self.config.get('browser'))
+        _browser_options = BROWSER_OPTIONS.get(self.config.get('browser', 'chrome'))
         _undetection_available = _browser_options.get('undetectable', False)
-        _undetectable = self.config.get('undetectable')
+        _undetectable = self.config.get('undetectable', False)
         _exec_path = _browser_options.get('exec_path')
         _remote = self.config.get('remote')
 
-        if _undetectable:
-            # Utilizar el driver indetectable
+        if _undetectable and _undetection_available:
             options = uc.ChromeOptions()
             options.add_argument('--no-sandbox')
             options.add_argument('--disable-blink-features=AutomationControlled')
-            options.add_argument('--disable-extensions')
+            options.add_argument('--disable-dev-shm-usage')
             options.add_argument('--disable-infobars')
-            if self.config.get("hidden"):
+            
+            options.add_argument('--disable-speech-api')
+            options.add_argument('--disable-features=VoiceInteraction')
+            options.add_argument('--disable-background-networking')
+            options.add_argument('--disable-default-apps')
+            options.add_argument('--disable-extensions')
+            options.add_experimental_option('excludeSwitches', ['enable-logging'])
+            
+            if self.config.get("hidden", False):
                 options.headless = True
             if _exec_path and os.path.isfile(_exec_path):
                 options.binary_location = _exec_path
             self.driver = uc.Chrome(options=options)
             logging.info("[DRIVER] Started in undetectable mode")
         else:
-            # Utilizar el driver estándar
             _webdriver = _browser_options.get('webdriver')
             _options = _browser_options.get('options')()
             _capabilities = _browser_options.get('capabilities')
 
             _options.add_argument('--no-sandbox')
+            
+            if _webdriver == webdriver.Chrome:
+                _options.add_argument('--disable-speech-api')
+                _options.add_argument('--disable-features=VoiceInteraction')
+                _options.add_argument('--disable-background-networking')
+                _options.add_argument('--disable-default-apps')
+                _options.add_experimental_option('excludeSwitches', ['enable-logging'])
 
-            if self.config.get("hidden"):
+            if self.config.get("hidden", False):
                 _options.add_argument("--headless")
             if _exec_path and os.path.isfile(_exec_path):
                 _options.binary_location = _exec_path
@@ -367,7 +428,6 @@ class PyWebWizardBase:
                     if service_path := _browser_options.get("service_path"):
                         os.popen(service_path)
                     _profile = webdriver.FirefoxProfile()
-                    # Configuración adicional para Firefox si es necesario
                     self.driver = _webdriver(firefox_profile=_profile, options=_options)
                 else:
                     self.driver = _webdriver(options=_options)
@@ -375,37 +435,6 @@ class PyWebWizardBase:
                 self.driver = _webdriver(options=_options)
             logging.info("[DRIVER] Started in detectable mode")
 
-    def _execute_action(self, action: dict):
-        """
-        Executes a single action.
-
-        :param action: Action dictionary.
-        """
-        action = self.prepare_action(action)
-        _action_name = action.get('action')
-        if _action_name not in self.actions_map:
-            raise ActionError(f"'{_action_name}' action not supported")
-        try:
-            return self.actions_map.get(_action_name)(action)
-        except TimeoutException as e:
-            if action.get('optional', False):
-                logging.info(f"[{_action_name.upper()}] Optional action timed out... Ignoring")
-                return None
-            else:
-                raise TimeoutException(f"Timeout in action '{_action_name}': {str(e)}")
-        except NoSuchElementException as e:
-            if action.get('optional', False):
-                logging.info(f"[{_action_name.upper()}] Optional element not found... Ignoring")
-                return None
-            else:
-                raise NoSuchElementException(f"Element not found in action '{_action_name}': {str(e)}")
-        except Exception as e:
-            if action.get('optional', False):
-                logging.info(f"[{_action_name.upper()}] Optional action failed... Ignoring")
-                return None
-            else:
-                raise
-
     def _execute_js(self, *args, **kwargs):
         """
         Executes a JavaScript script in the context of the current session.
@@ -428,7 +457,6 @@ class PyWebWizardBase:
             locator = ("xpath", query)
 
         try:
-            # Espera a que el elemento sea visible y esté interactuable
             element = wait.until(EC.element_to_be_clickable(locator))
             return element
         except TimeoutException:
@@ -445,9 +473,8 @@ class PyWebWizardBase:
         :param timeout: Maximum time to wait for elements.
         :return: Single web element or list of web elements.
         """
-        self.driver.switch_to.default_content()  # Asegurarse de estar en el contexto principal
+        self.driver.switch_to.default_content()
         _elements = self._get_elements(interface, query, timeout)
-        # Si el elemento está dentro de un iframe, cambiar el contexto
         if not _elements.is_displayed():
             iframes = self.driver.find_elements_by_tag_name('iframe')
             for iframe in iframes:
@@ -663,17 +690,20 @@ class PyWebWizard(PyWebWizardBase):
     """
 
     # Todo Acabar de adaptar para poder cargarle spell_book
-    def __init__(self, spell_file: str, spell_book: list = None, headers: dict = None,
+    def __init__(self, spell_file: str = None, spell_book: list = None, headers: dict = None,
                  destruct_on_finish: bool = True, external_functions_mapping: dict = None,
-                 keepass_file: str = None, keepass_password: str = None):
+                 keepass_file: str = None, keepass_password: str = None,
+                 openai_api_key: str = None, ai_model: str = "gpt-4"):
         """
         Initializes the PyWebWizard with a specific configuration file and optional headers.
 
         :param spell_file: Path to the Spell file.
         :param headers: Optional dictionary of headers for web requests.
+        :param openai_api_key: Optional OpenAI API key for AI-powered automation.
+        :param ai_model: AI model to use (default: gpt-4).
         """
         super().__init__(spell_file, headers, destruct_on_finish, external_functions_mapping,
-                         keepass_file, keepass_password)
+                         keepass_file, keepass_password, openai_api_key, ai_model)
 
     def get_actions(self):
         """
@@ -1036,6 +1066,11 @@ class PyWebWizard(PyWebWizardBase):
     @action_name('html')
     @required_fields([])
     def _html(self, action: dict):
+        """
+        Retrieves the HTML source of the current page.
+        
+        :param action: Dictionary containing the action parameters
+        """
         action = self.prepare_action(action)
         _response_variable = action.get('response')
         _debug = action.get('debug', False)
